import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import javax.swing.JTextArea;

// questa classe estende la JTextArea e la modifica per accettare solo testo binario
public class BinaryTextArea extends JTextArea implements KeyListener
{
	
	// costruttore
	public BinaryTextArea()
	{
		super(); // richiama il costruttore della superclasse
		
		this.addKeyListener(this); // aggiunge se stessa al KeyListener
	}
	
	@Override 
	// override del metodo richiamato quando si incolla del testo, necessario per controllare prima se  binario o meno
	public void paste() 
	{
		// verifica se  stato copiato del testo
		Toolkit tk = Toolkit.getDefaultToolkit();
		Transferable clipboardContent = tk.getSystemClipboard().getContents(null);
		if(clipboardContent!=null && (clipboardContent.isDataFlavorSupported (DataFlavor.stringFlavor))) // se  stato realmente copiato 
		{
		  try 
		  {
			  // lo copia in una stringa
			  String text = (String)clipboardContent.getTransferData(DataFlavor.stringFlavor);
			  
			  // e verifica se  binario
			  boolean valid = true;
			  for(int i=0; i<text.length() && valid; i++)
				  if(text.charAt(i) != '0' && text.charAt(i) != '1') valid = false;
			  
			  // se  valido, lo incolla richiamando il metodo della superclasse
			  if(valid) super.paste();
		  }
		  catch(Exception ex) // in caso di eccezioni
		  {
		    System.exit(1); // esce dal programma
		  }
		}
		
		// alla fine verifica quali bottoni possano essere abilitati o meno
		GUI.verifyButtons();
	}

	@Override
	public void keyPressed(KeyEvent e) 
	{
		// in caso di inserimento di testo non valido
		if(e.getKeyChar() != '0' && e.getKeyChar() != '1' &&
				!Character.isISOControl(e.getKeyChar()) && 
				e.getKeyCode() != KeyEvent.VK_RIGHT && 
				e.getKeyCode() != KeyEvent.VK_LEFT &&
				e.getKeyCode() != KeyEvent.VK_UP && 
				e.getKeyCode() != KeyEvent.VK_DOWN)
			this.setEditable(false); //si rende non editable, in modo da non memorizzarlo
		
		// alla fine verifica quali bottoni possano essere abilitati o meno
		GUI.verifyButtons();
	}

	@Override
	public void keyReleased(KeyEvent e) 
	{
		this.setEditable(true); //si rende editable al rilascio del tasto
		
		// verifica quali bottoni possano essere abilitati o meno
		GUI.verifyButtons();
	}

	@Override
	public void keyTyped(KeyEvent e) 
	{
		// verifica quali bottoni possano essere abilitati o meno
		GUI.verifyButtons();
	}
}
