// questa classe contiene tutti i metodi necessari per le varie conversioni e codifiche
public class Codify //gli input bit e controlName sono rispettivamente la lunghezza in bit dei char e la scelta di usare stringhe specifiche per i caratteri speciali o no
{
	public static String stringToBin(String s, int bit, boolean controlName) //converte la stringa s in binario
	{
		String tmp = ""; //la stringa convertita da ritornare
		for(int i=0; i<s.length(); i++) //scansiona tutta la stringa s
		{
			char c=' ';
			if(controlName) //controlla se  necessario effettuare una scansione maggiormente dettagliata
			{
				if(s.charAt(i)=='\\' && s.charAt(i+1)=='<') //inizio della stringa caratterizzante un carattere speciale
				{
					String tmp1 = ""; //stringa del carattere speciale
					i+=2;
					while(s.charAt(i)!='>') //scansiona fino alla fine della stringa del carattere speciale
					{
						tmp1+=s.charAt(i);
						i++;
					}
					c = specialCharecterIdentify(tmp1); //identifica il carattere speciale e lo memorizza nel carattere che andr ad aggiungersi alla stringa finale da tornare
				}
				else if(s.charAt(i)=='\\' && s.charAt(i+1)=='\\') //se incontra il carattere '\' due volte di seguito, deve memorizzarne solo uno in binario (controllare la UserGuide oppure l'help)
				{
					c = '\\';
					i++;
				}
				else c = s.charAt(i); // se non si  incontrato nessun carattere speciale si procede normalmente
			}
			else c = s.charAt(i); //in questo caso procede normalemente anche se si incontrano caratteri speciali
			
			tmp += charToBin(c, bit);
		}
		return tmp;
	}
	
	public static String charToBin(char c, int bit) //converte un char in binario
	{
		String tmp = "";
		for(int i=bit-1; i>=0; i--)
			if((c&(int)(Math.pow(2,i)))==0)
				tmp += 0;
			else tmp += 1;
		
		return tmp;
	}
	
	public static String binToString(String s, int bit, boolean controlName) //converte la stringa s da binario in testo normale
	{
		if(s.length()==0) return s;
		int cS=0;
		String tmp="";
		while(cS + bit <= s.length()) //scansiona tutto il testo binario
		{
			tmp += binToChar(s.substring(cS,cS + bit), bit, controlName); //converte char per char e memorizza nella stringa riornare
			cS += bit;
		}
		
		return tmp;
	}
	
	public static String binToChar(String bin, int bit, boolean controlName) //converte un char da binario a testo normale
	{
		int tmp1 = 0;
		byte tmp2;
		for(int i=0; i<bin.length(); i++) //calcola che il char bin  il tmp1-esimo carattere
		{
			tmp2 = Byte.parseByte("" + bin.charAt(i));
			tmp1 += Math.pow(2,bin.length()-i-1) * tmp2;
		}
		
		if(controlName && tmp1<=127 && Character.isISOControl((char)tmp1)) //se il carattere  speciale ed  abilitata l'opportuna scelta, verr tornata una stringa del genere: \<stringaCarattereSpeciale>
			return "\\<" + getISOControlName(tmp1) + ">";
		return ""+(char)tmp1; //altrimenti converte semplicemente l'intero in char
	}
	
	public static String codifyWithString(String s, String key, int bit) //codifica/decodifica la stringa s con la chiave key
	{
		if(s.length()==0 || key.length()==0) return s;
		int cKey=0, cS=0;
		while(cS + bit < s.length()) //scansiona la stringa s
		{
			s = s.substring(0,cS) + codifyWithChar(s.substring(cS,cS + bit), key.charAt(cKey), bit) + s.substring(cS + bit,s.length());
			cS += bit;
			cKey = (cKey + 1) % key.length(); //se la chiave finisce, ricomincia dal suo primo carattere
		}
		s = s.substring(0,cS) + codifyWithChar(s.substring(cS,cS + bit), key.charAt(cKey), bit); //il ciclo precedente non include l'ultimo char
		return s;
	}
	
	public static String codifyWithChar(String s, char k, int bit) //codifica/decodifica un char con un char della chiave
	{
		char c = (binToChar(s, bit, false)).charAt(0);
		c=(char)(c^k); //LA CODIFICA AVVIENE TRAMITE L'UTILIZZO DELLO XOR BIT A BIT
		return charToBin(c, bit);
	}
	
	public static String getISOControlName (int c) //ritornan la stringa caratteristica del carattere speciale numero c
	{
		switch(c)
		{
			case (0): return "NUL";
			case (1): return "SOH";
			case (2): return "STX";
			case (3): return "ETX";
			case (4): return "EOT";
			case (5): return "ENQ";
			case (6): return "ACK";
			case (7): return "BEL";
			case (8): return "BS";
			case (9): return "HT";
			case (10): return "FL";
			case (11): return "VT";
			case (12): return "FF";
			case (13): return "CR";
			case (14): return "SO";
			case (15): return "SI";
			case (16): return "DLE";
			case (17): return "DC1";
			case (18): return "DC2";
			case (19): return "DC3";
			case (20): return "DC4";
			case (21): return "NAK";
			case (22): return "SYN";
			case (23): return "ETB";
			case (24): return "CAN";
			case (25): return "EM";
			case (26): return "SUB";
			case (27): return "ESC";
			case (28): return "FS";
			case (29): return "GS";
			case (30): return "RS";
			case (31): return "US";
			case (127): return "DEL";
		}
		return "";
	}
	
	public static char specialCharecterIdentify (String s) //identifica il carattere speciale indicato dalla stringa caratteristica s
	{
		if(s.equals("NUL")) return 0;
		else if(s.equals("SOH")) return 1;
		else if(s.equals("STX")) return 2;
		else if(s.equals("ETX")) return 3;
		else if(s.equals("EOT")) return 4;
		else if(s.equals("ENQ")) return 5;
		else if(s.equals("ACK")) return 6;
		else if(s.equals("BEL")) return 7;
		else if(s.equals("BS")) return 8;
		else if(s.equals("HT")) return 9;
		else if(s.equals("FL")) return 10;
		else if(s.equals("VT")) return 11;
		else if(s.equals("FF")) return 12;
		else if(s.equals("CR")) return 13;
		else if(s.equals("SO")) return 14;
		else if(s.equals("SI")) return 15;
		else if(s.equals("DLE")) return 16;
		else if(s.equals("DC1")) return 17;
		else if(s.equals("DC2")) return 18;
		else if(s.equals("DC3")) return 19;
		else if(s.equals("DC4")) return 20;
		else if(s.equals("NAK")) return 21;
		else if(s.equals("SYN")) return 22;
		else if(s.equals("ETB")) return 23;
		else if(s.equals("CAN")) return 24;
		else if(s.equals("EM")) return 25;
		else if(s.equals("SUB")) return 26;
		else if(s.equals("ESC")) return 27;
		else if(s.equals("FS")) return 28;
		else if(s.equals("GS")) return 29;
		else if(s.equals("RS")) return 30;
		else if(s.equals("US")) return 31;
		else if(s.equals("DEL")) return 127;
		
		return(' ');
	}
	
	public static String setControlCharacter (String s, boolean actived) //converte la stringa s da normale all'utilizzo delle strighe caratteristiche dei caratteri speciali, se il booleano actived  true, viceversa se  false
	{
		String tmp="";
		if(actived) for(int i=0; i<s.length(); i++)
					{
						if(s.charAt(i)=='\\') tmp+="\\\\";
						else if((int)(s.charAt(i))<=127 && Character.isISOControl((char)s.charAt(i)))
							tmp += "\\<" + getISOControlName(s.charAt(i)) + ">";
						else tmp += s.charAt(i);
					}
		else for(int i=0; i<s.length(); i++)
			{
				if(s.charAt(i)=='\\' && s.charAt(i+1)=='<')
				{
					i+=2;
					String tmp1="";
					while(s.charAt(i)!='>')
					{
						tmp1+=s.charAt(i);
						i++;
					}
					tmp+=specialCharecterIdentify(tmp1);
				}
				else if(s.charAt(i)=='\\' && s.charAt(i+1)=='\\')
				{
					tmp+='\\';
					i++;
				}
				else tmp+=s.charAt(i);
			}
		return tmp;
	}
}