import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

// questa classe estende un ImagePanel e viene creato per la codifica passo passo, infatti contiene tutti gli elementi grafici e i metodi per tale codifica
public class CodifyPanel extends ImagePanel implements ActionListener
{
	
	private String binText, key, binKey, codifiedText;
	private int bit, textIndex, keyIndex;
	private JLabel textLabel;
	private JLabel [] charLabels, keyLabels, resLabels;
	private JButton prevStepButton, nextStepButton, completeCodificationButton;

	public CodifyPanel(String path, String _binText, String _key) 
	{
		super(path); // richiama il costruttore della superclasse
		
		bit = GUI.getBitNumber(); // ottiene dalla classe GUI il numero di bit selezionati
		
		// setta gli indici del testo da codificare e della chiave da usare
		textIndex = -bit;
		keyIndex = -1;
		
		codifiedText = ""; // crea le stringa che conterr il testo codificato
		
		binText = _binText; // salva il testo da codificare
		
		// crea e riempie l'array di stringhe dei caratteri in binario della chiave
		key = _key;
		binKey = Codify.stringToBin(key, bit, false); // testo della chiave in binario
		
		// crea i componenti della parte superiore della GUI
		textLabel = new JLabel(); // conterr il testo da codificare, la chiave e il testo codificato
		textLabel.setFont(new Font("Dialog", Font.BOLD, 13));
		textLabel.setOpaque(false);
		
		JScrollPane northScrollPane = new JScrollPane(textLabel);
		northScrollPane.setOpaque(false);
		
		JPanel northPanel = new JPanel();
		northPanel.setLayout(new BoxLayout(northPanel, BoxLayout.X_AXIS));
		northPanel.setOpaque(false);
		
		northPanel.add(Box.createRigidArea(new Dimension(15, 0)));
		northPanel.add(northScrollPane);
		northPanel.add(Box.createRigidArea(new Dimension(15, 0)));
		
		// crea i componenti della parte centrale della GUI
		JPanel centerPanel = new JPanel();
		centerPanel.setLayout(new BoxLayout(centerPanel, BoxLayout.X_AXIS));
		centerPanel.setOpaque(false);
		
		charLabels = new JLabel[bit]; // array dei JLabel dei caratteri da codificare
		keyLabels = new JLabel[bit]; // array dei JLabel dei caratteri della chiave
		resLabels = new JLabel [bit]; // array dei JLabel dei caratteri codificati
		
		JPanel [] xorGatePanels = new JPanel [bit]; // array di pannelli delle porte xor
		
		for(int i=0; i<xorGatePanels.length; i++)
		{
			xorGatePanels[i] = new JPanel(); // crea il pannello della porta xor i-esima
			xorGatePanels[i].setLayout(new BoxLayout(xorGatePanels[i], BoxLayout.Y_AXIS));
			xorGatePanels[i].setOpaque(false);
			
			charLabels[i] = new JLabel(); // crea il label dei caratteri da codificare
			charLabels[i].setFont(new Font("Dialog", Font.BOLD, 20));
			charLabels[i].setHorizontalAlignment(SwingConstants.LEFT);
			charLabels[i].setToolTipText(getOrdinalFor(bit-i-1) + " bit of character of the text"); // imposta il tooltip
			
			keyLabels[i] = new JLabel(); // crea il JLabel dei caratteri della chiave
			keyLabels[i].setFont(new Font("Dialog", Font.BOLD, 20));
			keyLabels[i].setHorizontalAlignment(SwingConstants.RIGHT);
			keyLabels[i].setToolTipText(getOrdinalFor(bit-i-1) + " bit of the character of the key"); // imposta il tooltip
			
			JPanel keyPanel = new JPanel(); // crea il pannello dove attaccare il keyLabel[i]
			keyPanel.setOpaque(false);
			keyPanel.setLayout(new BoxLayout(keyPanel, BoxLayout.Y_AXIS));
			keyPanel.add(Box.createRigidArea(new Dimension(0, 20)));
			keyPanel.add(keyLabels[i]);
			
			JPanel charAndkeyPanel = new JPanel(); // crea il pannello dovve attaccare charLabel[i] e keyLabel[i]
			charAndkeyPanel.setOpaque(false);
			charAndkeyPanel.setLayout(new BoxLayout(charAndkeyPanel, BoxLayout.X_AXIS));
			charAndkeyPanel.add(charLabels[i]);
			charAndkeyPanel.add(Box.createRigidArea(new Dimension((bit == 8)? 15 : 10, 0)));
			charAndkeyPanel.add(keyPanel);
			
			JLabel bitNumberLabel = new JLabel("" + (bit - i-1)); // crea il JLabel con il numero della porta xor
			bitNumberLabel.setFont(new Font("Dialog", Font.BOLD, 15));
			bitNumberLabel.setHorizontalAlignment(SwingConstants.CENTER);
			bitNumberLabel.setToolTipText(getOrdinalFor(bit-i-1) + " XOR gate"); // imposta il tooltip
			
			ImagePanel xorGateImage = new ImagePanel("xor.png");
			xorGateImage.setLayout(new GridLayout(1,1));
			xorGateImage.add(bitNumberLabel);
			
			resLabels[i] = new JLabel(); // crea il JLabel dei caratteri codificati
			resLabels[i].setFont(new Font("Dialog", Font.BOLD, 20));
			resLabels[i].setHorizontalAlignment(SwingConstants.CENTER);
			resLabels[i].setToolTipText(getOrdinalFor(bit-i-1) + " bit of the resoult"); // imposta il tooltip
			
			JPanel resPanel = new JPanel(); // crea il pannello dove attaccare il resLabel[i]
			resPanel.setOpaque(false);
			resPanel.setLayout(new BoxLayout(resPanel, BoxLayout.X_AXIS));
			resPanel.add(resLabels[i]);
			
			xorGatePanels[i].add(charAndkeyPanel);
			xorGatePanels[i].add(xorGateImage);
			xorGatePanels[i].add(resPanel);
			
			centerPanel.add(xorGatePanels[i]); // attacca al centerPanel il pannello della porta xor i-esima
		}
		
		// crea i componenti della parte inferiore della GUI
		JPanel southPanel = new JPanel();
		southPanel.setOpaque(false);
		
		prevStepButton = new JButton("Previous step"); // crea il bottone per retrocedere di step
		prevStepButton.addActionListener(this);
		
		nextStepButton = new JButton("Next step"); // crea il bottone per avanzare di step
		nextStepButton.addActionListener(this);
		
		completeCodificationButton = new JButton("Complete codification"); // crea il bottone per terminare la codifica
		completeCodificationButton.addActionListener(this);
		
		southPanel.add(prevStepButton);
		southPanel.add(nextStepButton);
		southPanel.add(completeCodificationButton);
		
		// attacca i pannelli creati a se stesso
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		
		this.add(Box.createRigidArea(new Dimension(0, 15)));
		this.add(northPanel);
		this.add(Box.createRigidArea(new Dimension(0, 15)));
		this.add(centerPanel);
		this.add(Box.createRigidArea(new Dimension(0, 15)));
		this.add(southPanel);
		
		nextStep();
	}
	
	// cambia il testo dei vari componenti in base allo step scelto (next o prev)
	private void changeStep()
	{
		String text = binText.substring(textIndex, textIndex + bit); // prende la porzione di testo da codificare
		String keyBin = binKey.substring(keyIndex*bit, keyIndex*bit+bit); // prende la porzione di chiave da utilizzare
		
		// modifica il testo dei label 
		for(int i=0; i<bit; i++)
		{
			charLabels[i].setText("" + text.charAt(i)); // dei caratteri da codificare,
			keyLabels[i].setText("" + keyBin.charAt(i)); // dei caratteri della chiave,
			resLabels[i].setText("" + Codify.codifyWithChar(text, key.charAt(keyIndex), bit).charAt(i)); // dei caratteri codificati
			
			codifiedText += resLabels[i].getText(); // aggiunge ogni volta il carattere codificato alola stringa del testo codificato
		}
		
		// aggiorna il label che contiene testo da codificare, chiave e testo codificato
		textLabel.setText("<html> Binary text: " + binText.substring(0, textIndex) + "<font size=5>" + binText.substring(textIndex, textIndex+bit) + "</font>" +  binText.substring(textIndex+bit) + 
				"<br> Key: " + binKey.substring(0, keyIndex*bit) + "<font size=5>" + binKey.substring(keyIndex*bit, keyIndex*bit+bit) + "</font>" + binKey.substring(keyIndex*bit+bit) + 
				"<br> Codified text: " + codifiedText.substring(0, textIndex) + "<font size=5>" + codifiedText.substring(textIndex, textIndex+bit) + "</font>" +  codifiedText.substring(textIndex+bit) + "</html>");
		
		// verifica se  possibile avanzare o retrocere di step, abilitando o disabilitando di conseguenza i bottoni
		if(textIndex >= binText.length() - bit)
			nextStepButton.setEnabled(false);
		else nextStepButton.setEnabled(true);
		
		if(textIndex < bit)
			prevStepButton.setEnabled(false);
		else prevStepButton.setEnabled(true);
		
		GUI.refreshFrame();
	}
	
	// retrocede di uno step
	private void prevStep()
	{
		textIndex -= bit; // diminuisce l'indice del testo del numero di bit
		keyIndex = (key.length()*bit + (keyIndex - 1)) % key.length(); // diminuisce l'indice della chiave, facendo attenzione alla circolarit della stessa
		
		codifiedText = codifiedText.substring(0, codifiedText.length() - 2 * bit); // elimina l'ultima parte del testo codificato
		
		changeStep(); // cambia lo step
	}
	
	// avanza di uno step
	private void nextStep()
	{
		textIndex +=  bit; // aumenta l'indice del testo del numero di bit
		keyIndex = (keyIndex + 1) % key.length(); // aumenta l'indice della chiave, facendo attenzione alla circolarit della stessa
		
		changeStep(); // cambia lo step
	}
	
	// ottiene il suffisso del numero ordinale in inglese
	private static String getOrdinalFor(int value) 
	{
		 switch (value) 
		 {
		 	case 1:
			   return value + "st";
			case 2:
			   return value + "nd";
			case 3:
			   return value + "rd";
			default:
			   return value + "th";
		 }
	}
	
	@Override
	public void actionPerformed(ActionEvent e) 
	{
		if(e.getActionCommand() == "Previous step") // se viene premuto il tasto "Previous step"
			prevStep(); // diminuisce di uno step
		if(e.getActionCommand() == "Next step") // se viene premuto il tasto "Next step"
			nextStep(); // avanza di uno step
		else if(e.getActionCommand() == "Complete codification") // se viene premuto il tasto "Complete codification"
			GUI.returnToBackground(); // torna semplicemente al pannello principale, in quanto era stata precedentemente
									// eseguita tutta la codifica nella classe GUI (subito dopo la creazione di questo pannello)
	}

}
