import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.event.*;

import javax.swing.JTextArea;

// questa classe estende la JTextArea, per poter implementare l'algoritmo per la visualizzazione o meno dei caratteri di controllo
public class ModifiedTextArea extends JTextArea implements KeyListener
{
	
	private boolean controlCharacter;
	
	// costruttore
	public ModifiedTextArea(boolean _controlCharacter)
	{
		super(); // richiama il costruttore della superclasse
		
		controlCharacter = _controlCharacter;
		this.addKeyListener(this); // aggiunge se stessa al KeyListener
	}
	
	// setta se vedere o meno i caratteri di controllo
	public void setControlCharacter(boolean _controlCharacter)
	{
		controlCharacter = _controlCharacter;
		
		this.setText(Codify.setControlCharacter(this.getText(), controlCharacter)); // imposta il testo
		
		verifyText(); // verifica il testo
	}
	
	// verifica se il testo  corretto o meno (in base alla codifica scelta per la visualizzazione dei caratteri di controllo)
	// '\\' per '\', '\<XX>' per il carattere XX
	// il carattere '\' non sar accettato, e dunque il metodo restituir false
	private boolean verifyTextControlCharacters(String text)
	{
		boolean valid = true;
		
		// viene anallizzato ogni carattere del testo
		for(int i=0; i<text.length() && valid; i++) 
		{
			if(text.charAt(i)=='\\') // se si incontra il carattere '\'
			{
				if(i == text.length() - 1) valid = false;
				else if(text.charAt(i+1)=='\\')
				{
					i++;
					valid = true;
				}
				else if(text.charAt(i+1)=='<')
				{
					i+=2;
					String tmp="";
					while(i<text.length() && text.charAt(i)!='>' && tmp.length()<4)
					{
						tmp+=text.charAt(i++);
					}
					if(i<text.length() && text.charAt(i)=='>')
						valid = (Codify.specialCharecterIdentify(tmp) != ' ');
					else valid = false;
				}
				else valid = false;
			}
			else if(text.charAt(i)<=127 && Character.isISOControl(text.charAt(i)))
			{
				String controlName = "\\<" + Codify.getISOControlName(text.charAt(i)) + ">";
				this.setText(text.substring(0, i) + controlName + text.substring(i+1));
				i += controlName.length() - 1;
			}
		}
		
		return valid;
	}
	
	// verifica il testo
	private void verifyText()
	{
		if(controlCharacter) // se si devono vedere i caratteri di controllo
		{
			// attiva o disattiva il bottone per la conversione da testo a binario e la spunta per abilitare o meno la visualizzazione dei caratteri di controllo,
			// in base se il testo  corretto o meno (in base alla codifica scelta per tale visualizzazione)
			boolean verified = verifyTextControlCharacters(this.getText());
			
 			GUI.setEnabledTextToBinAndIsoControlButton(verified);
		}
	}
	
	@Override 
	// override del metodo richiamato quando si incolla del testo, necessario per controllare prima se  binario o meno
	public void paste() 
	{
		super.paste();
		verifyText();
	}

	@Override
	public void keyPressed(KeyEvent e) 
	{
		verifyText();
	}

	@Override
	public void keyReleased(KeyEvent e) 
	{
		verifyText();
	}

	@Override
	public void keyTyped(KeyEvent e) 
	{
		verifyText();
	}
}
