import javax.swing.*;


import java.awt.*;
import java.awt.event.*;
import java.io.*;

// questa classe contiene tutti i metodi dell'interfaccia grafica del programma
public class GUI implements ActionListener
{
	private static ModifiedFrame frame;
	private static ImagePanel backgroundPanel;
	private static JPanel helpPanel;
	private static Container prevContentPane;
	private static ModifiedTextArea textArea;
	private static BinaryTextArea binTextArea;
	private static KeyTextField keyField;
	private static JCheckBox isoControlButton;
	private static JRadioButton bit8Button, bit16Button;
	private static JButton textLoadButton, textSaveButton, binTextLoadButton, binTextSaveButton,
						codifyButton, codifyStepButton, textToBinButton, binToTextButton;
	 
	public GUI()
	{
		// crea i vari componenti della GUI
		
		frame = new ModifiedFrame(); // crea il frame
		
		backgroundPanel = new ImagePanel("background.png"); // crea il pannello con l'immagine di sfondo
		backgroundPanel.setLayout(new BoxLayout(backgroundPanel, BoxLayout.X_AXIS));
		
		createTextPanel(); // crea il pannello textPanel che contiene la ModifiedTextArea e i bottoni del testo
		createBinTextPanel(); // crea il pannello binTextPanel che contiene la BinaryTextArea e i bottoni del testo binario
		createOptionPanel(); // crea il pannello optionPanel su cui attaccare il KeyTextField della chiave e i vari bottoni necessari per la codifica
		helpPanel = new HelpPanel("background.png"); // crea l'HelpPanel da visualizzare quando viene premuto il tasto Help della JMenuBar
		
		// imposta il contentPane del frame e lo rende visibile
		frame.setContentPane(backgroundPanel);
		frame.setVisible(true);
		
		// infine verifica quali bottoni possano essere abilitati o meno
		verifyButtons();
	}
	
	// crea il pannello textPanel che contiene la ModifiedTextArea e i bottoni del testo
	private void createTextPanel()
	{
		// crea il label del testo
		JLabel textLabel = new JLabel("Text:");
		textLabel.setHorizontalAlignment(JLabel.CENTER);
		textLabel.setFont(new Font("Dialog", Font.BOLD, 13));
		
		// crea la JTextArea modificata
		textArea = new ModifiedTextArea(false);
		textArea.setLineWrap(true);
		
		// crea lo ScrollPane che contiene la ModifiedTextArea
		JScrollPane textScrollPane = new JScrollPane(textArea);
		
		// crea i bottoni load e save
		textLoadButton = new JButton("Load");
		textLoadButton.setFont(new Font("Dialog", Font.BOLD, 13));
		textLoadButton.addActionListener(this);
		textSaveButton = new JButton("Save");
		textSaveButton.setFont(new Font("Dialog", Font.BOLD, 13));
		textSaveButton.addActionListener(this);
		
		// crea il pannello su cui attaccare i bottoni
		JPanel textButtonPanel = new JPanel();
		textButtonPanel.setOpaque(false);
		
		// attacca i bottoni
		textButtonPanel.add(textLoadButton);
		textButtonPanel.add(textSaveButton);
		
		// crea il pannello textPanel
		JPanel textPanel = new JPanel();
		textPanel.setLayout(new BorderLayout());
		textPanel.setOpaque(false);
		
		// attacca la ModifiedTextArea e il pannello dei bottoni al pannello appena creato
		textPanel.add(textLabel, BorderLayout.NORTH);
		textPanel.add(textScrollPane, BorderLayout.CENTER);
		textPanel.add(textButtonPanel, BorderLayout.SOUTH);
		
		// attacca il pannello allo sfondo
		backgroundPanel.add(Box.createRigidArea(new Dimension(17, 0)));
		backgroundPanel.add(textPanel);
	}
	
	// crea il pannello binTextPanel che contiene la BinaryTextArea e i bottoni del testo binario
	private void createBinTextPanel()
	{
		// crea il label testo binario
		JLabel binTextLabel = new JLabel("Binary text:");
		binTextLabel.setHorizontalAlignment(JLabel.CENTER);
		binTextLabel.setFont(new Font("Dialog", Font.BOLD, 13));
		
		// crea la JTextArea modificata per il testo binario
		binTextArea = new BinaryTextArea();
		binTextArea.setLineWrap(true);
		binTextArea.setEditable(true);
		
		// crea lo ScrollPane che contiene la BinaryTextArea
		JScrollPane binTextScrollPane = new JScrollPane(binTextArea);
						
		// crea i bottoni load e save
		binTextLoadButton = new JButton("Load");
		binTextLoadButton.setFont(new Font("Dialog", Font.BOLD, 13));
		binTextLoadButton.addActionListener(this);
		binTextSaveButton = new JButton("Save");
		binTextSaveButton.setFont(new Font("Dialog", Font.BOLD, 13));
		binTextSaveButton.addActionListener(this);
						
		// crea il pannello su cui attaccare i bottoni
		JPanel binTextButtonPanel = new JPanel();
		binTextButtonPanel.setOpaque(false);
				
		// attacca i bottoni
		binTextButtonPanel.add(binTextLoadButton);
		binTextButtonPanel.add(binTextSaveButton);
				
		// crea il pannello binTextPanel
		JPanel binTextPanel= new JPanel();
		binTextPanel.setLayout(new BorderLayout());
		binTextPanel.setOpaque(false);
						
		binTextPanel.add(binTextLabel, BorderLayout.NORTH);
		binTextPanel.add(binTextScrollPane, BorderLayout.CENTER);
		binTextPanel.add(binTextButtonPanel, BorderLayout.SOUTH);
				
		// attacca il pannello al background
		backgroundPanel.add(Box.createRigidArea(new Dimension(17, 0)));
		backgroundPanel.add(binTextPanel);
	}
	
	// crea il pannello optionPanel su cui attaccare il KeyTextField della chiave e i vari bottoni necessari per la codifica
	private void createOptionPanel()
	{
		// crea il label della chiave
		JLabel keyLabel = new JLabel("Insert the key and codify:", JLabel.LEFT);
		keyLabel.setFont(new Font("Dialog", Font.BOLD, 13));
		keyLabel.setIcon(new ImageIcon(new ImageIcon(ClassLoader.getSystemResource("key.png")).getImage().getScaledInstance(20, 20, Image.SCALE_DEFAULT)));
		
		// crea il JTextField modificato per contenere la chiave
		keyField = new KeyTextField(16);
		keyField.setFont(new Font("Dialog", Font.BOLD, 13));

		// crea il bottone per la codifica
		codifyButton = new JButton("Codify");
		codifyButton.setFont(new Font("Dialog", Font.BOLD, 13));
		codifyButton.addActionListener(this);
		
		// crea il bottone per la codifica passo dopo passo
		codifyStepButton = new JButton("Codify step by step");
		codifyStepButton.setFont(new Font("Dialog", Font.BOLD, 13));
		codifyStepButton.addActionListener(this);
		
		// crea vari pannelli su cui attaccare i vari componenti, per mantenere un giusto Layout
		JPanel optionKeyPanel = new JPanel();
		optionKeyPanel.setLayout(new GridLayout(4, 1, 0, 10));
		optionKeyPanel.setOpaque(false);
		
		optionKeyPanel.add(keyLabel);
		optionKeyPanel.add(keyField);
		optionKeyPanel.add(codifyButton);
		optionKeyPanel.add(codifyStepButton);
		
		// crea il pannello nord da attaccare all'optionPanel
		JPanel northOptionPanel = new JPanel();
		northOptionPanel.setOpaque(false);
		
		northOptionPanel.add(optionKeyPanel);
		
		// crea il JCheckBox per abilitare o meno la visualizzazione dei caratteri speciali
		isoControlButton = new JCheckBox("Enable control character name", true);
		isoControlButton.setFont(new Font("Dialog", Font.BOLD, 13));
		isoControlButton.setOpaque(false);
		isoControlButton.setSelected(false);
		isoControlButton.addActionListener(this);
		
		// crea i bottoni per selezionare il tipo di codifica binaria (8 o 16 bit)
		bit8Button = new JRadioButton("8 bit");
		bit8Button.setFont(new Font("Dialog", Font.BOLD, 13));
		bit8Button.setOpaque(false);
		bit8Button.setSelected(true);
		bit8Button.addActionListener(this);
		
		bit16Button = new JRadioButton("16 bit");
		bit16Button.setFont(new Font("Dialog", Font.BOLD, 13));
		bit16Button.setOpaque(false);
		bit16Button.addActionListener(this);
		
		ButtonGroup bg = new ButtonGroup();
		bg.add(bit8Button);
		bg.add(bit16Button);
		
		JPanel radioButtonPanel = new JPanel();
		radioButtonPanel.setOpaque(false);
		radioButtonPanel.add(bit8Button);
		radioButtonPanel.add(bit16Button);
		
		JPanel selectableOptionPanel = new JPanel();
		selectableOptionPanel.setOpaque(false);
		selectableOptionPanel.setLayout(new BorderLayout());
		selectableOptionPanel.add(isoControlButton, BorderLayout.NORTH);
		selectableOptionPanel.add(radioButtonPanel, BorderLayout.CENTER);
		
		// crea i bottoni per la conversione da testo a testo binario e viceversa
		textToBinButton = new JButton("Text to Binary");
		textToBinButton.setFont(new Font("Dialog", Font.BOLD, 13));
		textToBinButton.addActionListener(this);
		binToTextButton = new JButton("Binary to Text");
		binToTextButton.setFont(new Font("Dialog", Font.BOLD, 13));
		binToTextButton.addActionListener(this);
		
		// crea il pannello su cui attaccare i bottoni creati
		JPanel buttonOptionPanel = new JPanel();
		buttonOptionPanel.setLayout(new BoxLayout(buttonOptionPanel, BoxLayout.PAGE_AXIS));
		buttonOptionPanel.setOpaque(false);
		
		// attacca i bottoni
		buttonOptionPanel.add(Box.createRigidArea(new Dimension(60, 20)));
		buttonOptionPanel.add(textToBinButton);
		buttonOptionPanel.add(Box.createRigidArea(new Dimension(0, 20)));
		buttonOptionPanel.add(binToTextButton);
		
		// crea il pannello sud da attaccare all'optionPanel
		JPanel southOptionPanel = new JPanel();
		southOptionPanel.setLayout(new BorderLayout());
		southOptionPanel.setOpaque(false);
		
		southOptionPanel.add(selectableOptionPanel, BorderLayout.NORTH);
		southOptionPanel.add(buttonOptionPanel, BorderLayout.CENTER);
		
		// crea il pannello optionPanel
		JPanel optionPanel = new JPanel();
		optionPanel.setLayout(new GridLayout(2, 1));
		optionPanel.setOpaque(false);
		
		// attacca ad esso il pannello nord e quello sud
		optionPanel.add(northOptionPanel);
		optionPanel.add(southOptionPanel);
		
		// attacca il pannello al background
		backgroundPanel.add(optionPanel);
	}
	
	// viene richiamato quando viene premuto un tasto load, e apre una finestra per la selezione del file da caricare
	private void loadFile(JButton button) // deve essere passato il bottone che  stato premuto, in modo da verificare se il file da caricare deve essere binario o no
	{
		// crea il JFileChooser e gli applica il filtro per i file .txt
		JFileChooser fileChooser = new JFileChooser();
		
		fileChooser.setFileFilter(new ExtensionFileFilter(".txt", new String[] { "txt" }));
	    
		int status = fileChooser.showOpenDialog(null);
	    if (status == JFileChooser.APPROVE_OPTION) // se viene scelto un file
	    {
	      File selectedFile = fileChooser.getSelectedFile();
	      try
	      {
	    	  FileReader fr = new FileReader(selectedFile); // apre lo stream
	    	  
	    	  // viene resettata la textArea su cui caricare il testo
	    	  if(button == textLoadButton) textArea.setText("");
	    	  else binTextArea.setText("");
	    	  
	    	  // il do while cicla fino a quando non si arriva alla fine del file (carattere di controllo -1)
	    	  int n;
	    	  do
	    	  {
	    		  n = fr.read();
	    		  if(n != -1) // finch non si raggiunge il carattere di controllo
	    		  {
	    			  if(button == textLoadButton) //se  testo normale
	    				  textArea.append(""+(char)n); // vienne aggiunta alla textArea corretta il carattere successivo
	    			  else // altrimenti se il file deve essere binario
	    			  {
	    				  if((char)n != '0' && (char)n != '1') // se il carattere non  valido
	    				  {
	    					  // viene segnalato un errore
	    					  JOptionPane.showMessageDialog(null, "Invalid file", "Invalid file", JOptionPane.ERROR_MESSAGE);
	    					  binTextArea.setText("");
	    					  break;
	    				  }
	    				  else binTextArea.append(""+(char)n); // altrimenti aggiunta alla textArea corretta il carattere successivo
	    			  }
	    			  
	    		  }
	    		  
	    	  } while(n != -1);
	    	  
	    	  fr.close(); // chiude lo stream
	      }
	      catch(IOException ex) // se viene lanciata un'eccezione
	      {
	    	  System.exit(1); // esce dal programma
	      }
	    }
	}
	
	// viene richiamato quando viene premuto un tasto save, e apre una finestra per selezionare su quale file salvare
	private void saveFile(JButton button) // deve essere passato il bottone che  stato premuto, in modo da verificare se il file su cui salvare deve essere binario o no
	{
		// crea il JFileChooser e gli applica il filtro per i file .txt
		JFileChooser fileChooser = new JFileChooser();
		
		fileChooser.setFileFilter(new ExtensionFileFilter(".txt", new String[] { "txt" }));
	    
		int status = fileChooser.showSaveDialog(null);
	    if (status == JFileChooser.APPROVE_OPTION) // se viene scelto un file
	    {
	      File selectedFile = fileChooser.getSelectedFile();
	      
	      // se non viene inserita l'estensione manualmente, viene aggiunta in automatico
	      if(!selectedFile.getName().endsWith(fileChooser.getFileFilter().getDescription())) selectedFile = new File(selectedFile.getAbsolutePath() + fileChooser.getFileFilter().getDescription());
	      
	      try
	      {
	    	  FileWriter fw = new FileWriter(selectedFile); // apre lo stream
	    	  
	    	  // scrive su file il contenuto della textArea corretta
	    	  if(button == textSaveButton) fw.write(textArea.getText()); // quella del testo normale
	    	  else fw.write(binTextArea.getText()); // oppure quella del testo binario
	    	  
	    	  fw.flush();
	    	  fw.close(); // chiude lo stream
	      }
	      catch(IOException ex) // se viene lanciata un'eccezione
	      {
	    	  System.exit(1); // chiude il programma
	      }
	    }
	}
	
	@Override
	public void actionPerformed(ActionEvent e) 
	{
		if(e.getActionCommand() == "Load") // se viene premuto il tasto "Load"
			loadFile((JButton)e.getSource()); // richiama il metodo per aprire un file
		else if(e.getActionCommand() == "Save") // se viene premuto il tasto "Save"
			saveFile((JButton)e.getSource()); // richiama il metodo per salvare su un file
		else if(e.getActionCommand() == "Codify") // se viene premuto il tasto "Codify"
			binTextArea.setText(Codify.codifyWithString(binTextArea.getText(), keyField.getText(), getBitNumber())); // applica la codifica
		else if(e.getActionCommand() == "Codify step by step") // se viene premuto il tasto "Codify step by step"
		{
			// imposta come ContentPane del frame un nuovo CodifyPanel che si occuper dell'intera procedura
			frame.setContentPane(new CodifyPanel("background.png", binTextArea.getText(), keyField.getText()));
			binTextArea.setText(Codify.codifyWithString(binTextArea.getText(), keyField.getText(), getBitNumber())); // inoltre applica la codifica completa per aggiornare la binaryTextArea
		}
		else if(e.getActionCommand() == "Text to Binary") // se viene premuto il tasto "Text to Binary"
			binTextArea.setText(Codify.stringToBin(textArea.getText(), getBitNumber(), isoControlButton.isSelected())); // applica la codifica da testo a testo binario
		else if(e.getActionCommand() == "Binary to Text") // se viene premuto il tasto "Binary to text"
			textArea.setText(Codify.binToString(binTextArea.getText(), getBitNumber(), isoControlButton.isSelected())); // applica la codifica da testo binario a testo
		else if(e.getSource() == isoControlButton) // se viene messa o tolta la spunta dal tasto isoControlButton
			textArea.setControlCharacter(isoControlButton.isSelected()); // imposta nella ModifiedTextArea il valore booleano corretto
		
		verifyButtons();
		refreshFrame();
	}
	
	// verifica quali bottoni possano essere abilitati o meno
	public static void verifyButtons()
	{
		boolean enable = binTextArea.getText().length() % getBitNumber() == 0 && binTextArea.getText().length() > 0; // indica se il testo binario inserito  valido
		binToTextButton.setEnabled(enable); // e abilita o disabilita il bottone per la conversione binaria 
		
		enable = enable && keyField.getText().length() > 0; // inoltre verifica se  stata inserita una chiave
		codifyButton.setEnabled(enable); // abilita o disabilita il bottone per la codifica
		codifyStepButton.setEnabled(enable); // e abilita o disabilita il bottone per la codifica passo passo
	}
	
	// abilita o disabilita il bottone textToBinButton e isoControlButton
	// vengono disabilitati se  attivata la visualizzazione dei caratteri di controllo, ma sono stati inseriti codici non validi
	public static void setEnabledTextToBinAndIsoControlButton(boolean enabled)
	{
		textToBinButton.setEnabled(enabled);
		isoControlButton.setEnabled(enabled);
	}
	
	// applica un refresh al frame
	public static void refreshFrame()
	{
		frame.setVisible(true);
	}
	
	// ritorna il numero di bit in base al bottone selezionato
	public static int getBitNumber()
	{
		if(bit8Button.isSelected()) return 8;
		return 16;
	}
	
	// reimposta come ContentPane il pannello background
	public static void returnToBackground()
	{
		frame.setContentPane(backgroundPanel);
		refreshFrame();	
	}
	
	// apre o chiude il pannello dell'help
	public static void openCloseHelpPanel()
	{
		if(frame.getContentPane() == helpPanel) // se era gi aperto
			frame.setContentPane(prevContentPane); // imposta il pannello aperto in precedenza
		else
		{
			 prevContentPane = frame.getContentPane(); // altrimenti salva il pannello aperto in precedenza
			 frame.setContentPane(helpPanel); // e apre il pannello help
		}
		
		refreshFrame();
	}
	
}
