import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.URISyntaxException;

//questa classe estende un ImagePanel e viene creato per la visione dell'help, infatti contiene tutti gli elementi grafici e i metodi necessari
public class HelpPanel extends ImagePanel implements ActionListener
{
	
	private JButton backButton;
	
	public HelpPanel(String path) 
	{
		super(path); // richiama il costruttore della superclasse
		
		JLabel titleLabel = new JLabel("Help: "); // crea il JLabel con il titolo "Help:"
		titleLabel.setFont(new Font("Dialog", Font.BOLD, 13));
		
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		
		JTextArea textArea = new JTextArea(getTextFromFile("help.txt")); // crea la JTextArea con il testo preso dal file help.txt
		textArea.setFont(new Font("Dialog", Font.BOLD, 13));
		textArea.setLineWrap(true);
		textArea.setEditable(false);
		
		JScrollPane scrollPane = new JScrollPane(textArea); // crea il JScrollPane per contenere la JTextArea
		
		JPanel scrollPanePanel = new JPanel(); // crea il pannello per contenere il JScrollPane
		scrollPanePanel.setOpaque(false);
		scrollPanePanel.setLayout(new BoxLayout(scrollPanePanel, BoxLayout.X_AXIS));
		
		scrollPanePanel.add(Box.createRigidArea(new Dimension(25, 0)));
		scrollPanePanel.add(scrollPane);
		scrollPanePanel.add(Box.createRigidArea(new Dimension(25, 0)));
		
		backButton = new JButton("Back"); // crea il bottone per tornare indietro
		backButton.addActionListener(this);
		
		this.add(Box.createRigidArea(new Dimension(0, 10)));
		this.add(titleLabel);
		this.add(Box.createRigidArea(new Dimension(0, 10)));
		this.add(scrollPanePanel);
		this.add(Box.createRigidArea(new Dimension(0, 10)));
		this.add(backButton);
		this.add(Box.createRigidArea(new Dimension(0, 10)));
	}
	
	// ritorna il testo preso dal file 
	private String getTextFromFile(String fileName)
	{
		String text = ""; // stringa da ritornare
		
		try
		{
			BufferedReader br = new BufferedReader(new InputStreamReader(ClassLoader.getSystemResource(fileName).openStream())); // apre lo stream 
			
			String s;
			while((s = br.readLine()) != null) // legge una nuova linea del file
				text += s + "\n"; // e la aggiunge alla stringa da ritornare 
			br.close(); // chiude lo stream
			
		}
		catch(Exception e) // se viene lanciata un'eccezione
		{
			System.exit(1); // esce dal programma
		}
		
		return text; // RITORNA LA STRINGA
	}
	
	@Override
	public void actionPerformed(ActionEvent e) 
	{
		if(e.getActionCommand() == "Back") // se viene premuto il tasto "Back"
			GUI.openCloseHelpPanel(); // chiude il pannello Help
	}

}
