import java.awt.event.*;
import javax.swing.JTextField;

//questa classe estende la JTextField, modificandola per accettare un numero massimo di caratteri
public class KeyTextField extends JTextField implements KeyListener
{
	
	private final int MAX_LENGTH; // numero massimo di caratteri
	
	// costruttore
	KeyTextField()
	{
		this(16); // passa al costruttore in overload il valore di default 16 
	}
	
	// costruttore in overload
	public KeyTextField(int maxLength)
	{
		super(); // richiama il costruttore della superclasse
		
		MAX_LENGTH = maxLength; // imposta il numero massimo di caratteri 
		
		this.addKeyListener(this); // aggiunge se stessa al KeyListener
	}
	
	@Override
	// override del metodo richiamato quando si incolla del testo, necessario per controllare successivamente se la lunghezza  entro i limiti 
	public void paste()
	{
		super.paste(); // richiama il metodo della superclasse
		
		// verifica se la lunghezza del testo  entro i limiti
		if(this.getText().length() >= MAX_LENGTH)
			this.setText(this.getText().substring(0, MAX_LENGTH)); // in caso accorcia la stringa
		
		// alla fine verifica quali bottoni possano essere abilitati o meno
		GUI.verifyButtons();
	}
	
	@Override
	public void keyPressed(KeyEvent e) 
	{
		// in caso avesse raggiunto il numero massimo di caratteri
		if(this.getText().length() >= MAX_LENGTH &&  
				!Character.isISOControl(e.getKeyChar()) && 
				e.getKeyCode() != KeyEvent.VK_RIGHT && 
				e.getKeyCode() != KeyEvent.VK_LEFT)
			this.setEditable(false); //si rende non editable, in modo da non memorizzare pi alcun testo
		
		// alla fine verifica quali bottoni possano essere abilitati o meno
		GUI.verifyButtons();
	}

	@Override
	public void keyReleased(KeyEvent e) 
	{
		this.setEditable(true); //si rende editable al rilascio del tasto
		
		// verifica quali bottoni possano essere abilitati o meno
		GUI.verifyButtons();
	}

	@Override
	public void keyTyped(KeyEvent e) 
	{
		// verifica quali bottoni possano essere abilitati o meno
		GUI.verifyButtons();
	}

}
